import _titleize from "underscore.string/titleize";
import _compose from "recompact/compose";
import _withStateHandlers from "recompact/withStateHandlers";
import _withHandlers from "recompact/withHandlers";
import _withProps from "recompact/withProps";
import _isArray from "lodash/isArray";
import _times from "lodash/times";
import _last from "lodash/last";
import _each from "lodash/each";

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from 'react';
import { StyleSheet } from 'react-native';
import Screen from 'react-native-web-ui-components/Screen';
import Button from 'react-native-web-ui-components/Button';
import { getTitle, getComponent, FIELD_TITLE } from '../../utils';
import OrderHandle from './OrderHandle';
import RemoveHandle from './RemoveHandle';
import DraggableItem from './DraggableItem';
/* eslint react/no-array-index-key: 0 */

const styles = StyleSheet.create({
  label: {
    fontWeight: 'bold',
    paddingTop: 10
  },
  labelXs: {
    paddingBottom: 5
  }
});

const getItem = schema => {
  let newItem = '';

  if (schema.items.type === 'object') {
    newItem = {};
  } else if (schema.items.type === 'array') {
    newItem = [];
  }

  return newItem;
};

const formatTitle = title => _titleize(title).replace(/ies$/, 'y').replace(/s$/, '');

const iterateUiSchema = (uiSchema, i) => {
  const widgetProps = uiSchema['ui:widgetProps'] || {};
  const titleProps = uiSchema['ui:titleProps'] || {};
  const errorProps = uiSchema['ui:errorProps'] || {};
  return { ...uiSchema,
    'ui:widgetProps': _isArray(widgetProps) ? widgetProps[i] || {} : widgetProps,
    'ui:titleProps': _isArray(titleProps) ? titleProps[i] || {} : titleProps,
    'ui:errorProps': _isArray(errorProps) ? errorProps[i] || {} : errorProps
  };
};

const adjustUiSchema = (uiSchema, i) => {
  const adjustedUiSchema = iterateUiSchema(uiSchema, i);

  _each(uiSchema, (uis, key) => {
    if (!/^ui:/.test(key)) {
      adjustedUiSchema[key] = iterateUiSchema(uis, i);
    }
  });

  return adjustedUiSchema;
};

const getProps = ({
  name,
  value,
  schema,
  fields,
  uiSchema
}) => {
  const screenType = Screen.getType();
  const title = getTitle(uiSchema['ui:title'] || FIELD_TITLE, {
    name,
    value,
    key: _last(name.split('.'))
  });
  const propertySchema = schema.items;
  const propertyUiSchema = uiSchema.items;
  const PropertyField = getComponent(propertySchema.type, 'Field', fields);
  const options = uiSchema['ui:options'] || {};
  let adjustedUiSchema = uiSchema;
  const titleProps = uiSchema['ui:titleProps'] || {};
  titleProps.style = [styles.label, screenType === 'xs' ? styles.labelXs : null, titleProps.style || null];
  adjustedUiSchema = { ...uiSchema,
    'ui:titleProps': titleProps
  };
  const extraProps = {
    title,
    screenType,
    propertySchema,
    propertyUiSchema,
    PropertyField,
    uiSchema: adjustedUiSchema,
    addLabel: options.addLabel || `Add ${formatTitle(title)}`,
    addable: options.addable !== false,
    removable: options.removable !== false,
    orderable: options.orderable !== false,
    OrderComponent: options.OrderComponent || OrderHandle,
    RemoveComponent: options.RemoveComponent || RemoveHandle
  };
  return extraProps;
};

const onAddHandler = ({
  name,
  value,
  schema,
  onChange
}) => () => {
  const newItem = getItem(schema);
  const nextValue = value.concat(value.length !== 0 ? [newItem] : [newItem, newItem]);
  onChange(nextValue, name);
};

const onRemoveHandler = ({
  name,
  value,
  onChange,
  reorder,
  errors
}) => index => {
  const nextValue = value.filter((v, i) => i !== index);
  let nextErrors = errors;

  if (errors) {
    nextErrors = nextErrors.filter((v, i) => i !== index);
  }

  onChange(nextValue, name, {
    nextErrors: nextErrors || false
  });
  setTimeout(reorder);
};

const onItemRefHandler = ({
  refs
}) => (ref, index) => {
  refs[index] = ref; // eslint-disable-line
};

const PropertyComponentHandler = topProps => innerProps => React.createElement(DraggableItem, _extends({}, topProps, innerProps));

const ArrayWidget = _compose(_withStateHandlers({
  review: 0,
  dragging: null,
  refs: []
}, {
  setDragging: () => dragging => ({
    dragging
  }),
  reorder: ({
    review
  }) => () => ({
    review: review + 1,
    dragging: null,
    refs: []
  })
}), _withProps(getProps), _withHandlers({
  onAdd: onAddHandler,
  onRemove: onRemoveHandler,
  onItemRef: onItemRefHandler
}), _withHandlers({
  PropertyComponent: PropertyComponentHandler
}))(props => {
  const {
    review,
    name,
    value,
    title,
    theme,
    addLabel,
    addable,
    onAdd,
    widgets,
    schema,
    uiSchema,
    errors,
    dragging,
    screenType,
    propertyUiSchema,
    PropertyComponent
  } = props;
  const {
    LabelWidget
  } = widgets;
  const hasError = _isArray(errors) && errors.length > 0 && !errors.hidden;
  return React.createElement(React.Fragment, null, uiSchema['ui:title'] !== false ? React.createElement(LabelWidget, _extends({
    hasError: hasError,
    auto: uiSchema['ui:inline']
  }, uiSchema['ui:titleProps']), title) : null, schema.items.type === 'object' && uiSchema.items['ui:title'] !== false && screenType !== 'xs' ? React.createElement(PropertyComponent, _extends({}, props, {
    propertyName: `${name}.title`,
    propertyValue: getItem(schema),
    propertyErrors: {},
    propertyUiSchema: adjustUiSchema(propertyUiSchema, -1),
    index: -1,
    zIndex: 1,
    titleOnly: true
  })) : null, !value.length ? React.createElement(PropertyComponent, _extends({}, props, {
    key: `${review}.${name}.0`,
    propertyName: `${name}.0`,
    propertyValue: getItem(schema),
    propertyErrors: errors && errors[0],
    propertyUiSchema: adjustUiSchema(propertyUiSchema, 0),
    index: 0,
    zIndex: 1,
    noTitle: screenType !== 'xs'
  })) : null, _times(value.length, index => React.createElement(PropertyComponent, _extends({}, props, {
    key: `${review}.${name}.${index}`,
    propertyName: `${name}.${index}`,
    propertyValue: value[index],
    propertyErrors: errors && errors[index],
    propertyUiSchema: adjustUiSchema(propertyUiSchema, index),
    index: index,
    zIndex: dragging === index ? value.length : value.length - index,
    noTitle: screenType !== 'xs'
  }))), addable ? React.createElement(Button, {
    auto: true,
    small: true,
    flat: false,
    radius: true,
    type: theme.colors.primary,
    onPress: onAdd
  }, addLabel) : null);
});

export default ArrayWidget;